from enum import Enum


class StatusCodes(Enum):
    UNKNOWN = -1
    COMPLETED = 0
    FAILED = 1

    WAITING = 21
    IN_PROGRESS = 22

    NO_VERDICT = 41
    PARTIAL_VERDICT = 42

    TIMED_OUT = 61
    CANCELLED = 62
    INFRASTRUCTURE_ERROR = 63
    DATA_ERROR = 64

    PREREQUISITE_FAIL = 81
    API_CALL_FAILURE = 82
    NETWORK_FAILURE = 83

    DATA_VALIDATION_FAILURE = 84
    DEPENDENCY_UNAVAILABLE = 85
